// Copyright © 2011-2013, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package fi.jumi.core.suite;

import fi.jumi.core.api.SuiteListener;

import javax.annotation.concurrent.ThreadSafe;
import java.io.PrintStream;
import java.util.concurrent.Executor;

@ThreadSafe
class InternalErrorReportingExecutor extends InternalErrorReporter implements Executor {

    private final Executor backingExecutor;

    public InternalErrorReportingExecutor(Executor backingExecutor, SuiteListener listener, PrintStream out) {
        super(listener, out);
        this.backingExecutor = backingExecutor;
    }

    @Override
    public void execute(Runnable command) {
        backingExecutor.execute(new InternalErrorReportingCommand(command));
    }


    @ThreadSafe
    private class InternalErrorReportingCommand implements Runnable {
        private final Runnable command;

        public InternalErrorReportingCommand(Runnable command) {
            this.command = command;
        }

        @Override
        public void run() {
            try {
                command.run();
            } catch (Throwable t) {
                reportInternalError("Uncaught exception in thread " + Thread.currentThread().getName(), t);
            }
        }
    }
}
